from artiq.experiment import *
#from typing import TFloat

class UrukulCh(HasEnvironment):
    """Urukul single freq class

    Set the frecuencies/amplitudes of each Urukul channel
    """

    def build(self, ch: TInt32=0, freq: TFloat=100.0, amp: TFloat=1.0, name=None):
        self.ch = ch
        self.amp = amp
        self.freq = freq
        self.name = str(name) if name is not None else f'canal_{self.ch}'

        self.channel = self.get_device(f"urukul0_ch{self.ch}")

        ### This two attributes will be shown in the GUI grouped by channel
        # use/don't use each channel
        self.state = self.get_argument(f"state_ch{self.ch}", BooleanValue(1==0), name)
        # each channel's frequency
        self.frequency = self.get_argument(f"freq_ch{self.ch}",
                            NumberValue(self.freq*MHz, unit='MHz', scale=MHz, min=1*MHz, max=400*MHz),
                            name)
        # each channel's amplitude
        self.amplitude = self.get_argument(f"amp_ch{self.ch}",
                            NumberValue(self.amp, min=0., max=1.),
                            name)

    @rpc
    def generate_dataset(self):
        # UNTESTED
        self.set_dataset(f"laser_{self.name}_freq", self.frequency)
        self.set_dataset(f"laser_{self.name}_amp", self.amplitude)

    @kernel
    def initialize_channel(self):
        # initialises CPLD the selected channel
        #delay(100*ms)
        self.channel.cpld.init()
        #self.core.break_realtime()
        #delay(1000*ms)
        self.channel.init()

    @kernel
    def set_channel(self, profile: TInt32=0):
        self.channel.set(self.frequency, amplitude=self.amplitude, profile=profile)

        if self.state == True:
            self.channel.sw.on()
        else:
            self.channel.sw.off()

    @kernel
    def select_profile(self, number: TInt32):
        self.channel.cpld.set_profile(number)

    @kernel
    def set_frequency(self, freq: TFloat, amp: TFloat=0.2, profile: TInt32=0):
        #self.channel.set(freq, amplitude=self.amplitude, profile=profile)
        self.channel.set(freq, amplitude=amp, profile=profile)

    @kernel
    def set_amp(self, amp: TFloat, freq: TFloat, profile: TInt32=0):
        self.amplitude = amp
        self.channel.set(freq, amplitude=amp, profile=profile)

    def get_frequency(self) -> TFloat:
        return self.frequency

    @kernel
    def on(self):
        self.channel.sw.on()

    @kernel
    def off(self):
        self.channel.sw.off()

    @kernel
    def set_o(self, state: TBool):
        self.channel.sw.set_o(state)

    @kernel
    def pulse_mu(self, delay_time_mu: TInt64):
        self.channel.sw.pulse_mu(delay_time_mu)

    @kernel
    def pulse(self, delay_time: TFloat):
        self.channel.sw.pulse(delay_time)
